"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const actions_json_1 = __importDefault(require("./actions.json"));
const database_json_1 = __importDefault(require("./database.json"));
const config_1 = require("./types/config");
const Logger_1 = __importDefault(require("./utils/Logger"));
const matcher_1 = __importDefault(require("./utils/matcher"));
const Selector_1 = __importDefault(require("./utils/Selector"));
class App {
    constructor(rest) {
        this.rest = rest;
    }
    static filterData(replies, config) {
        return replies
            .filter(s => !config.favourited || s.favourited)
            .filter(s => s.favourites_count >= config.favourites)
            .filter(s => config.tagged.every(tag => s.tags.map(t => t.name).includes(tag)))
            .slice(0, config.last === -1 ? undefined : config.last);
    }
    run(rootId) {
        return __awaiter(this, void 0, void 0, function* () {
            this.load(rootId, this.process.bind(this));
        });
    }
    load(rootId, process) {
        return __awaiter(this, void 0, void 0, function* () {
            const { id: me } = yield this.login();
            const config = yield this.loadRoot(rootId, me);
            Logger_1.default.debug('Root config', config);
            if (!config.botodon) {
                Logger_1.default.error('Root status requires #botodon tag');
                return;
            }
            for (const action of yield this.loadActions(rootId, config.deep, config.file, config.shared ? undefined : me)) {
                const promise = process(me, action);
                if (!config.async) {
                    yield promise;
                }
            }
        });
    }
    login() {
        return __awaiter(this, void 0, void 0, function* () {
            const me = yield this.rest.getMe();
            Logger_1.default.debug(`Logged as ${me.acct}`);
            if (!me.bot) {
                Logger_1.default.warn('Please set account as bot !');
            }
            return me;
        });
    }
    loadRoot(id, me) {
        return __awaiter(this, void 0, void 0, function* () {
            const status = yield this.rest.getStatus(id);
            if (status.account.id !== me) {
                Logger_1.default.warn('Root status isn\'t yours');
            }
            return matcher_1.default({
                async: false,
                botodon: false,
                deep: false,
                shared: false,
                file: false
            }, status.tags.map(t => t.name).reverse());
        });
    }
    loadActions(id, deep, file, account) {
        return __awaiter(this, void 0, void 0, function* () {
            const lines = (yield this.rest.getReplies(id, deep, account))
                .map(s => ({ id: s.id, tags: s.tags.map(t => t.name) })).concat(file ? actions_json_1.default : []);
            Logger_1.default.debug(`Found ${lines.length} action(s)`);
            if (!lines.length) {
                Logger_1.default.error('Root status is empty !');
            }
            return lines;
        });
    }
    process(me, action) {
        var e_1, _a, e_2, _b, e_3, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const config = matcher_1.default({
                botodon: false,
                data: {
                    file: false,
                    from: [],
                    deep: false,
                    shared: false,
                    tagged: [],
                    favourited: false,
                    favourites: 0,
                    last: -1,
                    weighted: false,
                    same: false
                },
                global: false,
                followers: false,
                followers_of: [],
                replies: {
                    to: [],
                    deep: false,
                    visibility: false
                },
                favourites: [],
                visibility: 'unlisted'
            }, action.tags.reverse());
            Logger_1.default.debug(`Action ${action.id} config`, config);
            if (!config.botodon) {
                Logger_1.default.error(`Action status ${action.id} requires #botodon tag`);
                return;
            }
            if (!config_1.VISIBILITIES.includes(config.visibility)) {
                Logger_1.default.error(`Action status ${action.id}: invalid visibility ${config.visibility}`);
                return;
            }
            // Data
            const datas = (yield Promise.all(config.data.from.map(id => this.loadData(id, config.data, me))))
                .reduce((a, b) => a.concat(b), config.data.file ? this.loadDataFile(config.data) : []);
            if (!datas.length) {
                Logger_1.default.error(`Action ${action.id}: Any content`);
                return;
            }
            const selector = new Selector_1.default(datas, config.data.same, s => config.data.weighted ? s.favourites_count : 1);
            // Targets
            // TODO: progressive send (limit memory usage)
            const targets = [];
            if (config.global) {
                targets.push({ visibility: config.visibility });
            }
            if (config.followers) {
                config.followers_of.push(me);
            }
            try {
                for (var _d = __asyncValues(config.followers_of.map(id => this.rest.getFollowers(id))), _e; _e = yield _d.next(), !_e.done;) {
                    const followers = _e.value;
                    Logger_1.default.debug(followers);
                    targets.push(...followers.map(({ acct }) => ({ acct, visibility: config.visibility })));
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_e && !_e.done && (_a = _d.return)) yield _a.call(_d);
                }
                finally { if (e_1) throw e_1.error; }
            }
            try {
                for (var _f = __asyncValues(config.replies.to.map(id => this.rest.getReplies(id, config.replies.deep))), _g; _g = yield _f.next(), !_g.done;) {
                    const replies = _g.value;
                    targets.push(...replies.map(({ account: { acct }, visibility }) => ({
                        acct, visibility: config.replies.visibility ? visibility : config.visibility
                    })));
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (_g && !_g.done && (_b = _f.return)) yield _b.call(_f);
                }
                finally { if (e_2) throw e_2.error; }
            }
            try {
                for (var _h = __asyncValues(config.favourites.map(id => this.rest.getFavouritedBy(id))), _j; _j = yield _h.next(), !_j.done;) {
                    const fav = _j.value;
                    targets.push(...fav.map(({ acct }) => ({ acct, visibility: config.visibility })));
                }
            }
            catch (e_3_1) { e_3 = { error: e_3_1 }; }
            finally {
                try {
                    if (_j && !_j.done && (_c = _h.return)) yield _c.call(_h);
                }
                finally { if (e_3) throw e_3.error; }
            }
            Logger_1.default.debug(`Action ${action.id}: ${targets.length} target(s)`);
            if (!targets.length) {
                Logger_1.default.warn(`Action ${action.id}: Any target`);
            }
            for (const target of targets) {
                const next = selector.next();
                yield this.rest.postStatus({
                    media_ids: next.media_attachments.map(m => m.id),
                    sensitive: next.sensitive,
                    spoiler_text: next.spoiler_text,
                    status: `${target.acct ? `@${target.acct} ` : ''}${next.content.replace(/<[^>]*>?/gm, '')}`,
                    visibility: target.visibility
                });
            }
            Logger_1.default.debug(`Action ${action.id}: done`);
        });
    }
    loadData(id, config, me) {
        return __awaiter(this, void 0, void 0, function* () {
            return App.filterData(yield this.rest.getReplies(id, config.deep, config.shared ? undefined : me), config);
        });
    }
    loadDataFile(config) {
        return App.filterData(database_json_1.default.filter(s => config.deep || !s.deep).map(s => ({
            id: 'local', uri: 'file', account: { id: 'local', acct: 'file', bot: true, display_name: 'file', emojis: [] }, content: s.content || '',
            created_at: '', emojis: [], favourited: s.favourited || false, favourites_count: 0, media_attachments: (s.medias || []).map(m => ({
                id: m, description: '', url: 'file', preview_url: 'file', type: 'img'
            })), sensitive: false, reblogged: false, reblogs_count: 0,
            replies_count: 0, visibility: 'direct', tags: (s.tags || []).map(t => ({ name: t }))
        })), config);
    }
}
exports.default = App;
