"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Logger_1 = __importDefault(require("./utils/Logger"));
class Rest {
    constructor(api, limiter) {
        this.api = api;
        this.limiter = limiter;
    }
    getMe() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.get('accounts/verify_credentials');
        });
    }
    getStatus(id) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.get(`statuses/${id}`);
        });
    }
    getContext(id) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.get(`statuses/${id}/context`);
        });
    }
    getDescendants(id) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getContext(id)
                .then(context => context.descendants);
        });
    }
    getReplies(id, deep, account) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getDescendants(id).then(replies => replies
                .filter(s => deep || s.in_reply_to_id === id)
                .filter(s => !account || s.account.id === account));
        });
    }
    getFollowers(id) {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO: use link header
            return this.get(`accounts/${id}/followers`, { limit: 999 });
        });
    }
    getFavouritedBy(id) {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO: use link header
            return this.get(`statuses/${id}/favourited_by`, { limit: 999 });
        });
    }
    postStatus(status) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.post('statuses', status);
        });
    }
    deleteStatus(id) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.delete(`statuses/${id}`);
        });
    }
    postApp(redirectUri, scopes) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.post('/apps', {
                client_name: 'botodon',
                redirect_uris: redirectUri, scopes,
                website: 'https://git.wadza.fr/me/botodon'
            });
        });
    }
    call(promise) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.limiter.promise(promise.catch(err => {
                Logger_1.default.error(`Rest: ${err.message} on ${err.options.uri}`);
                throw err;
            }));
        });
    }
    get(url, qs = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.call(this.api.get(url, { qs }));
        });
    }
    post(url, body) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.call(this.api.post(url, { body }));
        });
    }
    delete(url) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.call(this.api.delete(url));
        });
    }
}
exports.default = Rest;
