"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Logger_1 = __importDefault(require("./Logger"));
class Limiter {
    constructor(calls, time, sync = false) {
        this.calls = calls;
        this.time = time;
        this.sync = sync;
        this.past = [];
        this.queue = [];
        this.timer = null;
    }
    dequeue() {
        const now = Date.now();
        this.past = this.past.filter(t => now - t < this.time);
        while (this.past.length < this.calls && this.queue.length > 0) {
            this.past.push(now);
            const action = this.queue.shift();
            action();
            if (this.sync) {
                break;
            }
        }
        if (this.queue.length <= 0) {
            this.timer = null;
        }
        else {
            const delay = this.sync ? this.time / this.calls : this.time - now + this.past[0];
            Logger_1.default.warn(`Limiter: waiting ${delay}ms...`);
            this.timer = setTimeout(this.dequeue.bind(this), delay);
        }
    }
    limit(callback) {
        this.queue.push(callback);
        if (this.timer === null) {
            this.timer = setImmediate(this.dequeue.bind(this));
        }
    }
    promise(promise) {
        let res = null;
        let rej = null;
        const wrapper = new Promise((resolve, reject) => {
            res = resolve;
            rej = reject;
        });
        this.limit(() => promise.then(res).catch(rej));
        return wrapper;
    }
}
exports.default = Limiter;
